#include "data_utility.h"
#include "data_struct.h"
#include "read_write.h"
#include "maxUtility.h"
#include "medianhull.h"
#include <vector>
#include <ctime>
using namespace std;

int main(int argc, char *argv[])
{ 
	//srand(time(0));
	
	// read the dataset
	point_set_t* P = read_points("2d.txt");
	point_set_t* skyline = skyline_point(P);
	int dim = P->points[0]->dim;
	
	// generate the utility vector
	point_t* u = alloc_point(dim);
	for (int i = 0; i < dim; i++)
	u->coord[i] = ((double)rand()) / RAND_MAX;

	// look for the ground truth maximum utility point
	int maxIdx = 0;
	double maxValue = 0;
	for(int i = 0; i < skyline->numberOfPoints; i++)
	{
		double value = dot_prod(u, skyline->points[i]);
		if(value > maxValue)
		{
			maxValue = value;
			maxIdx = i;
		}
	}
	printf("-----------------------------------------------\n");
	printf("|%15s |%15s |%10s |\n", "Method", "# of Questions", "Point #ID");
	printf("-----------------------------------------------\n");

	
	printf("|%15s |%15s |%10d |\n", "Ground Truth", "-", skyline->points[maxIdx]->id);
	printf("-----------------------------------------------\n");


	// parameters
	int s = 3;
	double epsilon = 0;
	int maxRound = 1000;
	double Qcount, Csize;
	point_t* p;
	int prune_option = RTREE;
	int dom_option = HYPER_PLANE;
	int stop_option = EXACT_BOUND;
	int cmp_option;
	
	// the 2-dimensional algorithms (only called when dim = 2)
	if (dim == 2)
	{
		// the Median algorithm
		p = medianVertices(skyline, u, maxRound, Qcount, Csize);
		printf("|%15s |%15.0lf |%10d |\n", "Median", Qcount, p->id);
		printf("-----------------------------------------------\n");
		
		// the Hull algorithm
		p = hullVertices(skyline, u, s, maxRound, Qcount, Csize);
		printf("|%15s |%15.0lf |%10d |\n", "Hull", Qcount, p->id);
		printf("-----------------------------------------------\n");
	}

	// the UH-Simplex algorithm
	cmp_option = SIMPLEX;
	p =  max_utility(skyline, u, s, epsilon, maxRound, Qcount, Csize, cmp_option, stop_option, prune_option, dom_option);
	printf("|%15s |%15.0lf |%10d |\n", "UH-Simplex", Qcount, p->id);
	printf("-----------------------------------------------\n");

	// the UH-Random algorithm
	cmp_option = RANDOM;
	p =  max_utility(skyline, u, s, epsilon, maxRound, Qcount, Csize, cmp_option, stop_option, prune_option, dom_option); 
	printf("|%15s |%15.0lf |%10d |\n", "UH-Random", Qcount, p->id);
	printf("-----------------------------------------------\n");

	release_point_set(skyline, false);
	release_point_set(P, true);

	return 0;	
}